<?php

if (!defined('ABSPATH')) exit;

use Minishlink\WebPush\WebPush;
use Minishlink\WebPush\Subscription;

if (!class_exists('daftplugInstantifyPwaAdminPushnotifications')) {
    class daftplugInstantifyPwaAdminPushnotifications {
        public $name;
        public $description;
        public $slug;
        public $version;
        public $textDomain;
        public $optionName;

        public $pluginFile;
        public $pluginBasename;

        public $settings;
        public $subscribedDevices;

        public function __construct($config) {
            $this->name = $config['name'];
            $this->description = $config['description'];
            $this->slug = $config['slug'];
            $this->version = $config['version'];
            $this->textDomain = $config['text_domain'];
            $this->optionName = $config['option_name'];

            $this->pluginFile = $config['plugin_file'];
            $this->pluginBasename = $config['plugin_basename'];

            $this->settings = $config['settings'];
            $this->vapidKeys = get_option("{$this->optionName}_vapid_keys", true);
            $this->subscribedDevices = get_option("{$this->optionName}_subscribed_devices", true);

            add_action("wp_ajax_{$this->optionName}_send_notification", array($this, 'doModalPush'));
        }

        public function doModalPush() {
            $imageId = $_POST['pushImage'];
            $imageUrl = '';

            if (get_post_type($imageId) == 'attachment') {
                $image = daftplugInstantifyPwa::resizeImage($imageId, 500, 500, true, '');
                if ($image) {
                    $imageUrl = $image[0];
                }
            }

            $data = array(
                'title' => sanitize_text_field($_POST['pushTitle']),
                'body' => sanitize_text_field($_POST['pushBody']),
                'url' => esc_url_raw($_POST['pushUrl']),
                'imageUrl' => esc_url_raw($imageUrl),
                'subscription' => sanitize_text_field($_POST['subscription']),
            );

            if (wp_verify_nonce($_POST['nonce'], "{$this->optionName}_send_notification_nonce")) {
                $sendNotification = $this->sendNotification($data);
            }

            wp_die($sendNotification);
        }

        public function sendNotification($data) {
            require_once plugin_dir_path($this->pluginFile) . implode(DIRECTORY_SEPARATOR, array('pwa', 'includes', 'libs', 'web-push-php', 'autoload.php'));

            $auth = array(
                'VAPID' => array(
                    'subject' => get_bloginfo('wpurl'),
                    'publicKey' => $this->vapidKeys['pwaPublicKey'],
                    'privateKey' => $this->vapidKeys['pwaPrivateKey'],
                ),
            );

            $webPush = new WebPush($auth);

            $subscriptionEndpoint = $data['subscription'];
            unset($data['subscription']);

            $badge = daftplugInstantify::getSetting('pwaNotificationBarIcon');

            if (get_post_type($badge) == 'attachment') {
                $badgeImg = daftplugInstantifyPwa::resizeImage($badge, 96, 96, true, '');
                if ($badgeImg) {
                    $badgeUrl = $badgeImg[0];
                }
            } else {
                $badgeUrl = '';
            }

            $data['icon'] = $data['imageUrl'];

            $data = shortcode_atts(array(
                'title' => '',
                'badge' => $badgeUrl,
                'body' => '',
                'icon' => '',
                'image' => '',
                'data' => array('url' => $data['url'])
            ), $data);

            if ($subscriptionEndpoint == 'all') {
                $subscriptions = array();
                foreach ($this->subscribedDevices as $subscription) {
                    $subscriptions[] =  array(
                                            'subscription' => Subscription::create(
                                                array(
                                                    'endpoint' => $subscription['endpoint'],
                                                    'publicKey' => $subscription['userKey'],
                                                    'authToken' => $subscription['userAuth'],
                                                )
                                            ),
                                            'payload' => null
                                        );
                }

                foreach ($subscriptions as $subscription) {
                    $webPush->sendNotification(
                        $subscription['subscription'],
                        json_encode($data)
                    );
                }
            } else {
                $subscription = array(
                                    'subscription' => Subscription::create(
                                        array(
                                            'endpoint' => $this->subscribedDevices[$subscriptionEndpoint]['endpoint'],
                                            'publicKey' => $this->subscribedDevices[$subscriptionEndpoint]['userKey'],
                                            'authToken' => $this->subscribedDevices[$subscriptionEndpoint]['userAuth'],
                                        )
                                    ),
                                    'payload' => null
                                );

                $webPush->sendNotification(
                    $subscription['subscription'],
                    json_encode($data)
                );
            }

            foreach ($webPush->flush() as $report) {
                $endpoint = $report->getRequest()->getUri()->__toString();
                if (!$report->isSuccess()) {
                    $subscribedDevices = $this->subscribedDevices;
                    unset($subscribedDevices[$endpoint]);
                    update_option("{$this->optionName}_subscribed_devices", $subscribedDevices);
                    //wp_die("[x] Message failed to sent for subscription {$endpoint}: {$report->getReason()}");
                }
            }

            wp_die('1');
        }
    }
}