<?php

if (!defined('ABSPATH')) exit;

if (!class_exists('daftplugInstantifyPwaPublicAddtohomescreen')) {
    class daftplugInstantifyPwaPublicAddtohomescreen {
    	public $name;
        public $description;
        public $slug;
        public $version;
        public $textDomain;
        public $optionName;

        public $pluginFile;
        public $pluginBasename;
        public $pluginUploadDir;
        public $pluginUploadUrl;

        public $settings;

        public $daftplugInstantifyPwaPublic;

    	public function __construct($config, $daftplugInstantifyPwaPublic) {
    		$this->name = $config['name'];
            $this->description = $config['description'];
            $this->slug = $config['slug'];
            $this->version = $config['version'];
            $this->textDomain = $config['text_domain'];
            $this->optionName = $config['option_name'];

            $this->pluginFile = $config['plugin_file'];
            $this->pluginBasename = $config['plugin_basename'];
            $this->pluginUploadDir = $config['plugin_upload_dir'];
            $this->pluginUploadUrl = wp_upload_dir()['baseurl'] . '/' . trailingslashit($config['slug']);

            $this->settings = $config['settings'];

            $this->daftplugInstantifyPwaPublic = $daftplugInstantifyPwaPublic;

            add_action('parse_request', array($this, 'generateManifest'));
            add_action('wp_head', array($this, 'renderManifestMetaTagsInHeader'), 1);
            add_action("wp_ajax_{$this->optionName}_save_installation_analytics", array($this, 'saveInstallationAnalytics'));
            add_action("wp_ajax_nopriv_{$this->optionName}_save_installation_analytics", array($this, 'saveInstallationAnalytics'));
			add_filter('query_vars', array($this, 'addManifestQueryVar'));
            add_shortcode('pwa-install-button', array($this, 'renderInstallationButton'));

            if (wp_is_mobile()) {
                if (daftplugInstantify::getSetting('pwaOrientation') !== 'any') {
                    add_filter("{$this->optionName}_public_html", array($this, 'renderRotateNotice'));
                }
            	if (daftplugInstantify::getSetting('pwaOverlays') == 'on') {
	                add_filter("{$this->optionName}_public_html", array($this, 'renderInstallationOverlays'));
	                if (in_array('menu', (array)daftplugInstantify::getSetting('pwaOverlaysTypes'))) {
	                    add_filter('wp_nav_menu_items', array($this, 'renderMenuOverlay'), 10, 2);
	                }
	                if (in_array('header', (array)daftplugInstantify::getSetting('pwaOverlaysTypes'))) {
	                    add_filter("{$this->optionName}_public_html", array($this, 'renderHeaderOverlay'));
	                }
	                if (in_array('checkout', (array)daftplugInstantify::getSetting('pwaOverlaysTypes'))) {
	                   add_action('woocommerce_review_order_after_payment', array($this, 'renderCheckoutOverlay'));
	                }
            	}
            }
    	}
        
        public function generateManifest() {
            if (isset($GLOBALS['wp']->query_vars['daftplugInstantifyPwaManifest'])) {
                if (1 == $GLOBALS['wp']->query_vars['daftplugInstantifyPwaManifest']) {
                    header('Content-Type: text/javascript; charset=utf-8');
                    $manifest = array();
                    $manifest['name'] = daftplugInstantify::getSetting('pwaName');

                    if (strlen(daftplugInstantify::getSetting('pwaShortName')) > 12) {
                        $manifest['short_name'] = substr(daftplugInstantify::getSetting('pwaShortName'), 0, 9).'...';
                    } else {
                        $manifest['short_name'] = daftplugInstantify::getSetting('pwaShortName');
                    }

                    $manifest['start_url'] = daftplugInstantify::getSetting('pwaStartPage');
                    $manifest['description'] = daftplugInstantify::getSetting('pwaDescription');
                    $manifest['display'] = daftplugInstantify::getSetting('pwaDisplayMode');
                    $manifest['orientation'] = daftplugInstantify::getSetting('pwaOrientation');
				    $manifest['dir'] = is_rtl() ? 'rtl' : 'ltr';
                    $manifest['background_color'] = daftplugInstantify::getSetting('pwaBackgroundColor');
                    $manifest['theme_color'] = daftplugInstantify::getSetting('pwaThemeColor');

					if (get_bloginfo('language')) {
						$manifest['lang'] = get_bloginfo('language');
					}

                    $icon = (has_site_icon()) ? get_option('site_icon') : daftplugInstantify::getSetting('pwaIcon');
                    $iconWidth = wp_get_attachment_image_src($icon, 'full')[1];
                    $iconSizes = array(180, 192, 512);

                    if (wp_attachment_is_image(intval($icon))) {
                        foreach ($iconSizes as $iconSize) {
                            if ($iconWidth < $iconSize) {
                                continue;
                            }

                            $newIcon = daftplugInstantifyPwa::resizeImage($icon, $iconSize, $iconSize, true, 'png');

                            if ($newIcon[1] != $iconSize) {
                                continue;
                            }

                            $manifest['icons'][] = array(
                                'src'   => $newIcon[0],
                                'sizes' => "{$iconSize}x{$iconSize}",
                                'type'  => 'image/png',
                            );
                        }
                    }

                    $manifest = apply_filters("{$this->optionName}_pwa_manifest", $manifest);
        			$manifest = json_encode($manifest, JSON_UNESCAPED_SLASHES);
					echo $manifest;
					exit;
                }
            }
        }

		public function addManifestQueryVar($queryVars) {
			$queryVars[] = 'daftplugInstantifyPwaManifest';

			return $queryVars;
		}

        public function renderManifestMetaTagsInHeader() {
            include_once($this->daftplugInstantifyPwaPublic->partials['manifestMetaTags']);
        }

        public function renderRotateNotice() {
            if (function_exists('is_amp_endpoint') && is_amp_endpoint()) {
                return;
            }
            
            include_once($this->daftplugInstantifyPwaPublic->partials['rotateNotice']);
        }

        public function renderInstallationOverlays() {
            if (function_exists('is_amp_endpoint') && is_amp_endpoint()) {
                return;
            }
            
            include_once($this->daftplugInstantifyPwaPublic->partials['installationOverlays']);
        }

        public function renderMenuOverlay($items, $args) {
            if (function_exists('is_amp_endpoint') && is_amp_endpoint()) {
                return;
            }

            $appIcon = (has_site_icon()) ? get_site_icon_url(150) : wp_get_attachment_image_src(daftplugInstantify::getSetting('pwaIcon'), array(150, 150))[0];
            $message = esc_html__('Find what you need faster with our web app!', $this->textDomain);
			$backgroundColor = daftplugInstantify::getSetting('pwaOverlaysBackgroundColor');
			$textColor = daftplugInstantify::getSetting('pwaOverlaysTextColor');
            $notNow = esc_html__('Not now', $this->textDomain);
            $install = esc_html__('Install', $this->textDomain);

            $items.= '<div class="daftplugPublic" data-daftplug-plugin="'.$this->optionName.'">
                          <div class="daftplugPublicMenuOverlay" style="background: '.$backgroundColor.'; color: '.$textColor.';">
                              <div class="daftplugPublicMenuOverlay_content">
                                  <img class="daftplugPublicMenuOverlay_icon" src="'.$appIcon.'">
                                  <span class="daftplugPublicMenuOverlay_msg">'.$message.'</span>
                              </div>
                              <div class="daftplugPublicMenuOverlay_buttons">
                                  <div class="daftplugPublicMenuOverlay_dismiss" style="color: '.$textColor.'99;">'.$notNow.'</div>
                                  <div class="daftplugPublicMenuOverlay_install" style="background: '.$textColor.'; color: '.$backgroundColor.';">'.$install.'</div>
                              </div>
                          </div>
                      </div>';

            return $items;
        }

        public function renderHeaderOverlay() {
            if (function_exists('is_amp_endpoint') && is_amp_endpoint()) {
                return;
            }
            
            include_once($this->daftplugInstantifyPwaPublic->partials['headerOverlay']);
        }

        public function renderCheckoutOverlay() {
            if (function_exists('is_amp_endpoint') && is_amp_endpoint() || !daftplugInstantify::isWooCommerceActive()) {
                return;
            }
            
            include_once($this->daftplugInstantifyPwaPublic->partials['checkoutOverlay']);
        }

        public function renderInstallationButton($atts) {
            if (function_exists('is_amp_endpoint') && is_amp_endpoint() || !wp_is_mobile() || daftplugInstantify::getSetting('pwaOverlays') !== 'on') {
                return;
            }

            $backgroundColor = daftplugInstantify::getSetting('pwaOverlaysBackgroundColor');
            $textColor = daftplugInstantify::getSetting('pwaOverlaysTextColor');
            $install = esc_html__('Install', $this->textDomain);

            $installButton = '<div class="daftplugPublic" data-daftplug-plugin="'.$this->optionName.'">
                                <button class="daftplugPublicInstallButton" style="background: '.$backgroundColor.'; color: '.$textColor.';">'.$install.'</button>
                              </div>';

            return $installButton;
        }

        public function saveInstallationAnalytics() {
            $data = get_transient("{$this->optionName}_installation_analytics");
            $data[(date('j M Y'))] += 1;
            set_transient("{$this->optionName}_installation_analytics", $data, 604800);

            wp_die();
        }

        public function getManifestUrl($encoded = true) {
            $url = add_query_arg(array('daftplugInstantifyPwaManifest' => 1), site_url('/', 'https'));
            if ($encoded) {
                return wp_json_encode($url);
            }

            return $url;
        }
    }
}